`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    12:16:54 02/25/2011 
// Design Name: 
// Module Name:    adc_config 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
module adc_config(
    input CLK,         // Clock
    input RST,         // Reset
    input INIT,        // Command to initialize the ADC's (on power up or on command)
	 input JCTRL,       // Selects JWE and JDATA as source for writing memory
    input JWE,          // JTAG Write enable for the memory
    input [25:0] JDATA,// Parallel data to be written to memory from JTAG
	 input [3:0] CSP_WE, // Write enables from chip scope pro control
	 input [23:0] CSP_WR_DATA, // Data to be written to memory from chip scope pro
	 input [4:0] CSP_WR_ADDR, // Address to write to memory from chip scope pro
	 input CSP_RD_CTRL,       // Selects CSP_RD_ADDR as source of address
	 output reg [23:0] CSP_RD_DATA, // Data to be written to memory from chip scope pro
	 input [4:0] CSP_RD_ADDR, // Address to write to memory from chip scope pro
	 input [1:0] CSP_MSEL, // memory select from chip scope pro
	 input [11:0] MASK, // Mask for which ADC to talk to
    output [11:0] CS,  // Chip selects for all 12 ADC's
    output reg SCLK,       // Serial clock to ADC's
    output SDATA,      // Serial data to ADC's
    output DONE,        // Done signal when initialization is complete
	 output [1:0] la_msel,
	 output [23:0] la_mem_out,
	 output [4:0] la_rd_addr,
	 output [4:0] la_wr_addr
    );


	reg [23:0] ram0 [31:0];
	reg [23:0] ram1 [31:0];
	reg [23:0] ram2 [31:0];
	reg [23:0] ram3 [31:0];
	reg [23:0] shreg;
	reg [23:0] mem_in;
	reg [3:0] we;
	reg [23:0] mem_out;
	wire [23:0] mem_out0;
	wire [23:0] mem_out1;
	wire [23:0] mem_out2;
	wire [23:0] mem_out3;
	reg [1:0] msel;
   reg [4:0] ja;
   reg [4:0] a;


	wire ms_rst;
	wire ms_inc;
	wire add_rst;
	wire cntr_rst;
	wire incr;
	wire load;
	wire scken;
	wire shen;
	wire csel;
	reg  dscken;
	reg [11:0] cs_mask;
	reg [4:0] raddr;
	reg [4:0] addr;
	reg [5:0] shift_cntr;
	
	initial begin
	   $readmemh ("ADC_ram2_contents", ram0, 0, 31); //We don't have four options now so set them all to ram2
	   $readmemh ("ADC_ram2_contents", ram1, 0, 31);
	   $readmemh ("ADC_ram2_contents", ram2, 0, 31);
	   $readmemh ("ADC_ram2_contents", ram3, 0, 31);
	end

	assign la_msel = msel;
	assign la_mem_out = mem_out;
	assign la_rd_addr = raddr;
	assign la_wr_addr = a;
	

   assign mem_out0 = ram0[raddr];
   assign mem_out1 = ram1[raddr];
   assign mem_out2 = ram2[raddr];
   assign mem_out3 = ram3[raddr];
	assign SDATA = shreg[23];
	assign csel = scken | dscken; // stretch the clock enable by 1 clock
	assign CS = {12{csel}} & cs_mask;
	
	always @(posedge CLK)  dscken <= scken;
	
   always @(msel or MASK) begin
	   case(msel)
		   2'd0: cs_mask = MASK & 12'b000000000101;
		   2'd1: cs_mask = MASK & 12'b000000001010;
		   2'd2: cs_mask = MASK & 12'b000011110000;
		   2'd3: cs_mask = MASK & 12'b111100000000;
			default: cs_mask = MASK & 12'b000000000011;
		endcase
	end
	
//
// Decode address and write enable to write from JTAG data
//
	always @(JDATA) begin
		case(JDATA[23:16])
			8'h00: ja = 5'h00;
			8'h01: ja = 5'h01;
			8'h0f: ja = 5'h02;
			8'h11: ja = 5'h03;
			8'h12: ja = 5'h04;
			8'h14: ja = 5'h05;
			8'h24: ja = 5'h06;
			8'h25: ja = 5'h07;
			8'h26: ja = 5'h08;
			8'h27: ja = 5'h09;
			8'h2a: ja = 5'h0a;
			8'h2b: ja = 5'h0b;
			8'h42: ja = 5'h0c;
			8'h45: ja = 5'h0d;
			8'h46: ja = 5'h0e;
			8'he2: ja = 5'h0f;
			default: ja = 5'h1f;
		endcase
	end
	
//
// Data/addr/we source multiplexer for memory writes
//
   always @(JCTRL or JDATA or JWE or ja or CSP_WE or CSP_WR_DATA or CSP_WR_ADDR) begin
	   if(JCTRL)
		   begin
				case(JDATA[25:24])
					2'd0: we = {1'b0,1'b0,1'b0,JWE};
					2'd1: we = {1'b0,1'b0,JWE,1'b0};
					2'd2: we = {1'b0,JWE,1'b0,1'b0};
					2'd3: we = {JWE,1'b0,1'b0,1'b0};
					default: we = 4'h0;
				endcase
				mem_in = JDATA[23:0];
				a = ja;
			end
		else
			begin
				we = CSP_WE;
				mem_in = CSP_WR_DATA;
				a = CSP_WR_ADDR;
			end
	end
	
//
// source multiplexer for memory reads
//
   always @* begin
	   if(CSP_RD_CTRL)
		   begin
				raddr = CSP_RD_ADDR;
				case(CSP_MSEL)
					2'd0: CSP_RD_DATA = mem_out0;
					2'd1: CSP_RD_DATA = mem_out1;
					2'd2: CSP_RD_DATA = mem_out2;
					2'd3: CSP_RD_DATA = mem_out3;
					default: CSP_RD_DATA = mem_out0;
				endcase
			end
		else
			begin
				raddr = addr;
				CSP_RD_DATA = mem_out;
			end
	end
	
	
//
// Infer distributed RAM for storing ADC configuration memory
//
   always @(posedge CLK) begin
	   if(we[0])
		   ram0[a] <= mem_in;
	end
   always @(posedge CLK) begin
	   if(we[1])
		   ram1[a] <= mem_in;
	end
   always @(posedge CLK) begin
	   if(we[2])
		   ram2[a] <= mem_in;
	end
   always @(posedge CLK) begin
	   if(we[3])
		   ram3[a] <= mem_in;
	end
	
//
// Mux for memory selection
//
   always @(msel or mem_out0 or mem_out1 or mem_out2 or mem_out3) begin
	   case(msel)
		   2'd0: mem_out = mem_out0;
		   2'd1: mem_out = mem_out1;
		   2'd2: mem_out = mem_out2;
		   2'd3: mem_out = mem_out3;
			default: mem_out = mem_out0;
		endcase
	end
//
// Shift register for serial data to ADCs 
//
	always @(posedge CLK) begin
	  if (load)
	     shreg <= mem_out;              // load with value from memory
	  else
	    if(shen & SCLK)
		    shreg <= {shreg[22:0],1'b0}; // shift left
		 else
		    shreg <= shreg;
	end
	
//
// SCLK generator (lives high)
//
	always @(posedge CLK) begin
	   if(!scken)
		   SCLK <= 1'b1;  // When not enabled the clock lives high
	   else
		   SCLK <= ~SCLK; // The serial clock is half the frequency of CLK
	end
	
//
// Shift counter with synchronous reset
//    counts CLK ticks not actual shifts
//
   always @(posedge CLK) begin
	   if(cntr_rst)
		   shift_cntr <= 6'h00;
	   else
		   if(scken)
			   shift_cntr <= shift_cntr + 1;
         else
            shift_cntr <= shift_cntr;
   end



//
// Address counter with synchronous reset
//
   always @(posedge CLK) begin
	   if(add_rst)
		   addr <= 5'h00;
	   else
		   if(incr)
			   addr <= addr + 1;
         else
            addr <= addr;
   end

//
// Memory selection counter with synchronous reset
//
   always @(posedge CLK) begin
	   if(ms_rst)
		   msel <= 2'd0;
	   else
		   if(ms_inc)
			   msel <= msel + 1;
         else
            msel <= msel;
   end

	
//
//  Finite State Machine for controlling the ADC configuration
//

   ADC_Config_FSM
   ADC_Config_FSM1	(
  .ADDR_RST(add_rst),    // output -- synchronous address counter reset
  .CNTR_RST(cntr_rst),   // output -- synchronous shift counter reset
  .MS_RST(ms_rst),       // output -- memory select counter reset
  .MS_INC(ms_inc),       // output -- memory select counter increment
  .DONE(DONE),           // output -- Conifguration Done signal
  .INCR(incr),           // output -- Increment address counter
  .LOAD(load),           // output -- Load shift register
  .SCKEN(scken),         // output -- Enable the serial clock generation and ADC chip select signals (CS)
  .SHEN(shen),           // output -- Shift Enable
  .ADR(addr),            // input  -- memory address
  .CLK(CLK),             // input  -- clock
  .INIT(INIT),           // input  -- Signal to initiate the configuration
  .RST(RST),             // input  -- Reset 
  .MSEL(msel),           // input  -- Memory select counter
  .SCNTR(shift_cntr));   // input  -- Shift counter for sequence control.

endmodule


// Created by fizzim.pl version 3.60 on 2011:04:13 at 16:03:16 (www.fizzim.com)

module ADC_Config_FSM (
  output reg ADDR_RST,
  output reg CNTR_RST,
  output reg DONE,
  output reg INCR,
  output reg LOAD,
  output reg MS_INC,
  output reg MS_RST,
  output reg SCKEN,
  output reg SHEN,
  input wire [4:0] ADR,
  input CLK,
  input INIT,
  input wire [1:0] MSEL,
  input RST,
  input wire [5:0] SCNTR 
);
  
  // state bits
  parameter 
  Idle         = 3'b000, 
  Done         = 3'b001, 
  End_Seq      = 3'b010, 
  Incr_Addr    = 3'b011, 
  Load         = 3'b100, 
  Next_mem     = 3'b101, 
  Shift_Enable = 3'b110, 
  Start_Seq    = 3'b111; 
  
  reg [2:0] state;
  reg [2:0] nextstate;
  
  // comb always block
  always @* begin
    nextstate = 3'bx; // default to x because default_state_is_x is set
    ADDR_RST = 0; // default
    CNTR_RST = 0; // default
    INCR = 0; // default
    LOAD = 0; // default
    MS_INC = 0; // default
    MS_RST = 0; // default
    SCKEN = 0; // default
    SHEN = 0; // default
    case (state)
      Idle        : begin
                                                      ADDR_RST = 1;
                                                      CNTR_RST = 1;
                                                      MS_RST = 1;
        if      (INIT)                                nextstate = Start_Seq;
        else                                          nextstate = Idle;
      end
      Done        : begin
                                                      ADDR_RST = 1;
                                                      CNTR_RST = 1;
                                                      MS_RST = 1;
        if      (~INIT)                               nextstate = Idle;
      end
      End_Seq     : begin
                                                      CNTR_RST = 1;
                                                      SCKEN = 1;
        if      (MSEL == 2'd3)                        nextstate = Done;
        else                                          nextstate = Next_mem;
      end
      Incr_Addr   : begin
                                                      CNTR_RST = 1;
                                                      INCR = 1;
                                                      SCKEN = 1;
                                                      SHEN = 1;
                                                      nextstate = Load;
      end
      Load        : begin
                                                      LOAD = 1;
                                                      SCKEN = 1;
                                                      SHEN = 1;
                                                      nextstate = Shift_Enable;
      end
      Next_mem    : begin
                                                      ADDR_RST = 1;
                                                      MS_INC = 1;
                                                      nextstate = Start_Seq;
      end
      Shift_Enable: begin
                                                      SCKEN = 1;
                                                      SHEN = 1;
        if      ((SCNTR == 6'd46) && (ADR == 5'h0f))  nextstate = End_Seq;
        else if ((SCNTR == 6'd46) && (ADR < 5'h0f))   nextstate = Incr_Addr;
        else                                          nextstate = Shift_Enable;
      end
      Start_Seq   : begin
                                                      LOAD = 1;
                                                      SCKEN = 1;
                                                      nextstate = Shift_Enable;
      end
    endcase
  end
  
  // Assign reg'd outputs to state bits
  
  // sequential always block
  always @(posedge CLK or posedge RST) begin
    if (RST)
      state <= Idle;
    else
      state <= nextstate;
  end
  
  // datapath sequential always block
  always @(posedge CLK or posedge RST) begin
    if (RST)     DONE <= 0;
    else begin
      DONE <= 0; // default
      case (nextstate)
        Done        : DONE <= 1;
      endcase
    end
  end
  
  // This code allows you to see state names in simulation
  `ifndef SYNTHESIS
  reg [95:0] statename;
  always @* begin
    case (state)
      Idle        : statename = "Idle";
      Done        : statename = "Done";
      End_Seq     : statename = "End_Seq";
      Incr_Addr   : statename = "Incr_Addr";
      Load        : statename = "Load";
      Next_mem    : statename = "Next_mem";
      Shift_Enable: statename = "Shift_Enable";
      Start_Seq   : statename = "Start_Seq";
      default     : statename = "XXXXXXXXXXXX";
    endcase
  end
  `endif

endmodule

