`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    15:45:37 08/26/2011 
// Design Name: 
// Module Name:    daq_op_sim_top 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
module daq_op_sim_top(
    input SYS_RST,
    input CMS_CLK,
    input REF_CLK,
	 input L1A,
	 input L1A_PHASE
    );

wire clk40,clk80,clk20,clk20_180,clk10,clk160,clk120;
wire daq_mmcm_rst;
wire daq_mmcm_lock;

  daq_mmcm daq_mmc1(.CLK_IN1(CMS_CLK),.CLK_OUT1(clk40),.CLK_OUT2(clk80),.CLK_OUT3(clk20),.CLK_OUT4(clk20_180),.CLK_OUT5(clk10),.CLK_OUT6(clk160),.CLK_OUT7(clk120),.RESET(daq_mmcm_rst),.LOCKED(daq_mmcm_lock));


 /////////////////////////////////////////////////////////////////////////////
 //                                                                         //
 //  Readout FIFO2 to feed MAC for GbE optical path.                        //
 //                                                                         //
 /////////////////////////////////////////////////////////////////////////////
	
	wire daq_data_clk;
	wire daq_data_clk_lock;
	wire [15:0] mactxd;
	wire mactxdvld;
	wire txack;
	reg [11:0] rdf_wdata;
	reg rdf_wren;
	reg [5:0] rst_ext;
	wire rdf_rst;
	wire [6:0] nsamp = 7'd8;
   wire [6:0] samp_max;
	assign samp_max = nsamp-1;
	assign rdf_rst = rst_ext[5];
	
	always @(posedge clk120 or posedge SYS_RST)
	begin
		if(SYS_RST)
			rst_ext <= 6'b111111;
		else
			rst_ext <= {rst_ext[4:0],SYS_RST || !daq_data_clk_lock};
	end

// Fill FIFO with incrementing numbers
	always @(posedge clk120 or posedge rdf_rst)
	begin
		if(rdf_rst)
			rdf_wdata <= 12'h000;
		else
			if(rdf_wdata < 12'd800)
				rdf_wdata <= rdf_wdata+1;
			else
				rdf_wdata <= rdf_wdata;
	end
	always @* begin
		if(rdf_rst)
			rdf_wren = 0;
		else
			if(rdf_wdata >= 12'd800)
				rdf_wren = 0;
			else
				rdf_wren =1;
	end
	
mac_fifo #(
	.USE_MAC_SRC_DEST(1)
	)
rd_fifo2(
   .WCLK(clk120),
   .RCLK(daq_data_clk),
   .CMSCLK(clk40),
   .RST(SYS_RST || !daq_data_clk_lock),
	.SAMP_MAX(samp_max),
   .WDATA(rdf_wdata),
	.WREN(rdf_wren),
	.L1A(L1A),
	.L1A_PHASE(L1A_PHASE),
	.TXACK(txack),                           // Data acknowledge signal from MAC processor
	.MACTXD(mactxd),                         // 16-bit data for MAC processor
	.MACTXDVLD(mactxdvld)                    // MAC data valid signal
   );


 /////////////////////////////////////////////////////////////////////////////
 //                                                                         //
 //  DAQ Path Optical Transceiver signals and GTX signals                   //
 //                                                                         //
 /////////////////////////////////////////////////////////////////////////////

   wire daq_rx_sd;

	daq_transceiver_io #(
		.USE_CHIPSCOPE(0),
		.SIM_SPEEDUP(1)
	)		
	daq_xcvr_io1 (
	   .RST(SYS_RST),
		// External signals
		.DAQ_SIGDET(1'b0),                 // receiver signal detected
		.DAQ_RX_N(1'b1),.DAQ_RX_P(1'b0), // high speed serial input
		.DAQ_TDIS(),                     // disable optical transmission
		.DAQ_TX_N(DAQ_TX_N),.DAQ_TX_P(DAQ_TX_P), // high speed serial output
		// Internal signals
		.DAQ_TX_REFCLK(REF_CLK),       // 125 MHz for GbE
		.MACTXD(mactxd),                         // 16-bit data for MAC processor
		.MACTXDVLD(mactxdvld),                   // MAC data valid signal
		.TXACK(txack),                           // Data acknowledge signal from MAC processor
		.DAQ_SD(daq_rx_sd),
		.DAQ_DATA_CLK(daq_data_clk),
		.DAQ_DATA_CLK_LOCK(daq_data_clk_lock)
  );

endmodule
