//----------------------------------------------------------------------
// Title      : Demo Testbench
// Project    : Virtex-6 Embedded Tri-Mode Ethernet MAC Wrapper
// File       : demo_tb.v
// Version    : 1.4
//-----------------------------------------------------------------------------
//
// (c) Copyright 2009-2010 Xilinx, Inc. All rights reserved.
//
// This file contains confidential and proprietary information
// of Xilinx, Inc. and is protected under U.S. and
// international copyright and other intellectual property
// laws.
//
// DISCLAIMER
// This disclaimer is not a license and does not grant any
// rights to the materials distributed herewith. Except as
// otherwise provided in a valid license issued to you by
// Xilinx, and to the maximum extent permitted by applicable
// law: (1) THESE MATERIALS ARE MADE AVAILABLE "AS IS" AND
// WITH ALL FAULTS, AND XILINX HEREBY DISCLAIMS ALL WARRANTIES
// AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY, INCLUDING
// BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-
// INFRINGEMENT, OR FITNESS FOR ANY PARTICULAR PURPOSE; and
// (2) Xilinx shall not be liable (whether in contract or tort,
// including negligence, or under any other theory of
// liability) for any loss or damage of any kind or nature
// related to, arising under or in connection with these
// materials, including for any direct, or any indirect,
// special, incidental, or consequential loss or damage
// (including loss of data, profits, goodwill, or any type of
// loss or damage suffered as a result of any action brought
// by a third party) even if such damage or loss was
// reasonably foreseeable or Xilinx had been advised of the
// possibility of the same.
//
// CRITICAL APPLICATIONS
// Xilinx products are not designed or intended to be fail-
// safe, or for use in any application requiring fail-safe
// performance, such as life-support or safety devices or
// systems, Class III medical devices, nuclear facilities,
// applications related to the deployment of airbags, or any
// other applications that could lead to death, personal
// injury, or severe property or environmental damage
// (individually and collectively, "Critical
// Applications"). Customer assumes the sole risk and
// liability of any use of Xilinx products in Critical
// Applications, subject only to applicable laws and
// regulations governing limitations on product liability.
//
// THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS
// PART OF THIS FILE AT ALL TIMES.
//
//----------------------------------------------------------------------
// Description: This testbench will exercise the PHY ports of the EMAC
//              to demonstrate the functionality.
//----------------------------------------------------------------------

`timescale 1ps / 1ps


module testbench;

  //--------------------------------------------------------------------
  // Testbench signals
  //--------------------------------------------------------------------
  wire        reset;

  wire        tx_client_clk;
  wire [7:0]  tx_ifg_delay;
  wire        rx_client_clk;
  wire [15:0] pause_val;
  wire        pause_req;

  // 1000BASE-X PCS/PMA wires
  wire        txp;
  wire        txn;
  wire        rxp;
  wire        rxn;
  wire        sync_acq_status;
  wire        [4:0] phyad;

  // Clock wires
  wire        host_clk;
  reg         gtx_clk;
  wire        mgtclk_p;
  wire        mgtclk_n;


  //----------------------------------------------------------------
  // Testbench Semaphores
  //----------------------------------------------------------------
  wire        configuration_busy;
  wire        monitor_finished_1g;
  wire        monitor_finished_100m;
  wire        monitor_finished_10m;

  // Tie off PHY address to a default value
  assign phyad = 5'b00001;

  //----------------------------------------------------------------
  // Wire up device under test
  //----------------------------------------------------------------
  DMB_MAC_example_design dut
    (
    // Client receiver interface
    .EMACCLIENTRXDVLD         (),
    .EMACCLIENTRXFRAMEDROP    (),
    .EMACCLIENTRXSTATS        (),
    .EMACCLIENTRXSTATSVLD     (),
    .EMACCLIENTRXSTATSBYTEVLD (),

    // Client transmitter interface
    .CLIENTEMACTXIFGDELAY     (tx_ifg_delay),
    .EMACCLIENTTXSTATS        (),
    .EMACCLIENTTXSTATSVLD     (),
    .EMACCLIENTTXSTATSBYTEVLD (),

    // MAC Control interface
    .CLIENTEMACPAUSEREQ       (pause_req),
    .CLIENTEMACPAUSEVAL       (pause_val),

    .EMACCLIENTSYNCACQSTATUS  (sync_acq_status),
    .EMACANINTERRUPT          (),

    // 1000BASE-X PCS/PMA interface
    .TXP                      (txp),
    .TXN                      (txn),
    .RXP                      (rxp),
    .RXN                      (rxn),
    .PHYAD                    (phyad),

    // Transceiver differential reference clock
    .MGTCLK_P                 (mgtclk_p),
    .MGTCLK_N                 (mgtclk_n),

    // Asynchronous reset
    .RESET                    (reset)
  );


  //--------------------------------------------------------------------------
  // Flow control is unused in this demonstration
  //--------------------------------------------------------------------------
  assign pause_req = 1'b0;
  assign pause_val = 16'b0;

  // IFG stretching not used in demo.
  assign tx_ifg_delay = 8'b0;


  //--------------------------------------------------------------------------
  // Clock drivers
  //--------------------------------------------------------------------------

  // Drive GTX_CLK at 125 MHz
  initial
  begin
    gtx_clk <= 1'b0;
    #10000;
    forever
    begin
      gtx_clk <= 1'b0;
      #4000;
      gtx_clk <= 1'b1;
      #4000;
    end
  end

  // Drive serial transceiver differential clock at 125MHz
  assign mgtclk_p = gtx_clk;
  assign mgtclk_n = ~gtx_clk;


  //--------------------------------------------------------------------
  // Instantiate the PHY stimulus and monitor
  //--------------------------------------------------------------------

  phy_tb phy_test
    (
      .clk125m               (gtx_clk),

      //----------------------------------------------------------------
      // GMII interface
      //----------------------------------------------------------------
      .txp                   (txp),
      .txn                   (txn),
      .rxp                   (rxp),
      .rxn                   (rxn),

   
      //----------------------------------------------------------------
      // Testbench semaphores
      //----------------------------------------------------------------
      .configuration_busy    (configuration_busy),
      .monitor_finished_1g   (monitor_finished_1g),
      .monitor_finished_100m (monitor_finished_100m),
      .monitor_finished_10m  (monitor_finished_10m),
      .monitor_error         (monitor_error)
    );


  //--------------------------------------------------------------------
  // Instantiate the no-host configuration stimulus
  //--------------------------------------------------------------------

  configuration_tb config_test
    (
      .reset                 (reset),

      //----------------------------------------------------------------
      // Host interface: host_clk is always required
      //----------------------------------------------------------------
      .host_clk              (host_clk),

      //----------------------------------------------------------------
      // Testbench semaphores
      //----------------------------------------------------------------

      .sync_acq_status       (sync_acq_status),

      .configuration_busy    (configuration_busy),
      .monitor_finished_1g   (monitor_finished_1g),
      .monitor_finished_100m (monitor_finished_100m),
      .monitor_finished_10m  (monitor_finished_10m),
      .monitor_error         (monitor_error)
    );

endmodule
