`timescale 1ns / 1ps


module reset_manager(
    input STUP_CLK,
    input CLK,
    input EOS,
    input JTAG_SYS_RST,
	 input CSP_SYS_RST,
    input DAQ_MMCM_LOCK,
    input TRG_MMCM_LOCK,
    input TRG_TX_PLL_LOCK,
    input TRG_TXRESETDONE,
    input QP_ERROR,
    input QP_LOCKED,
	 input ADC_INIT_DONE,
	 output ADC_INIT,
    output QP_RST_B,
    output TRG_GTXTXRESET,
    output MMCM_RST,
    output SYS_MON_RST,
    output ADC_RST,
    output LCT_RST,
	 output DSR_RST,
    output SYS_RST,
	 output POR,
	 output QPLL_LOCK,
	 output QPLL_ERROR,
	 output [2:0] POR_STATE
    );

//wire por;
wire adc_rdy;
wire ainc;
reg [2:0] awcnt;

wire rst_qpll;
  
 IBUF IBUF_QP_ERROR (.O(QPLL_ERROR),.I(QP_ERROR));
 IBUF IBUF_QP_LOCKED (.O(QPLL_LOCK),.I(QP_LOCKED));
 OBUF  #(.DRIVE(12),.IOSTANDARD("DEFAULT"),.SLEW("SLOW")) OBUF_QP_RST (.O(QP_RST_B),.I(~rst_qpll));
 

assign SYS_RST    = (JTAG_SYS_RST || CSP_SYS_RST || POR);
assign DSR_RST    = ~adc_rdy || SYS_RST;
assign SYS_MON_RST = 1'b0;

Pow_on_Rst       POW_on_Reset_FSM   (.MMCM_RST(MMCM_RST),.POR(POR),.RST_QPLL(rst_qpll),.POR_STATE(POR_STATE), // outputs
                                     .CLK(STUP_CLK),.EOS(EOS),.MMCM_LOCK(DAQ_MMCM_LOCK),.QPLL_LOCK(QPLL_LOCK)); // inputs
Trg_Clock_Strt   Trg_Clock_Strt_FSM (.GTX_RST(TRG_GTXTXRESET),.TRG_RST(LCT_RST), // outputs
                                     .CLK(STUP_CLK),.MMCM_LOCK(TRG_MMCM_LOCK),.POR(POR),.RST_DONE(TRG_TXRESETDONE)); // inputs

ADC_Init         ADC_Init_FSM       (.INC(ainc),.INIT(ADC_INIT),.RST(ADC_RST),.RUN(adc_rdy),
                                     .CLK(CLK),.CNT(awcnt),.INIT_DONE(ADC_INIT_DONE),.POR(POR));

always @(posedge CLK or posedge ADC_RST) begin
   if(ADC_RST)
	   awcnt <= 3'b000;
	else
	   if(ainc)
		   awcnt <= awcnt +1;
		else
		   awcnt <= awcnt;
end

endmodule


////////////////////////////////////////////////////////////////
//                                                            //
// Power on Reset signal State machine        (Uses STUP_CLK) //
//                                                            //
////////////////////////////////////////////////////////////////

module Pow_on_Rst (
  output reg MMCM_RST,
  output reg POR,
  output reg RST_QPLL,
  output [2:0] POR_STATE,
  input CLK,
  input EOS,
  input MMCM_LOCK,
  input QPLL_LOCK 
);
  
  // state bits
  parameter 
  Qpll_rst   = 3'b000, 
  Pow_on_Rst = 3'b001, 
  Run_State  = 3'b010, 
  W4Qpll     = 3'b011, 
  W4SysClk   = 3'b100; 
  
localparam 	POR_tmo = 4'hC;

  
  reg [2:0] state;
  reg [2:0] nextstate;
  reg [3:0] timer;
  reg por_cnt;
  wire por_done;
  
  assign POR_STATE = state;
  // comb always block
  always @* begin
    nextstate = 3'bx; // default to x because default_state_is_x is set
    case (state)
      Qpll_rst  :                       nextstate = W4Qpll;
      Pow_on_Rst: if      (!MMCM_LOCK)  nextstate = W4Qpll;
                  else if (por_done)    nextstate = Run_State;
                  else                  nextstate = Pow_on_Rst;
      Run_State :                       nextstate = Run_State;
      W4Qpll    : if      (QPLL_LOCK)   nextstate = W4SysClk;
                  else                  nextstate = W4Qpll;
      W4SysClk  : if      (MMCM_LOCK)   nextstate = Pow_on_Rst;
                  else                  nextstate = W4SysClk;
    endcase
  end
  
  // Assign reg'd outputs to state bits
  
  // sequential always block
  always @(posedge CLK or negedge EOS) begin
    if (!EOS)
      state <= Qpll_rst;
    else
      state <= nextstate;
  end
  
  // datapath sequential always block
  always @(posedge CLK or negedge EOS) begin
    if (!EOS) begin
      MMCM_RST <= 1;
      POR <= 1;
      RST_QPLL <= 1;
      por_cnt <= 0;
    end
    else begin
      MMCM_RST <= 0; // default
      POR <= 1; // default
      RST_QPLL <= 0; // default
      por_cnt <= 0; // default
      case (nextstate)
        Qpll_rst  : begin
                           MMCM_RST <= 1;
                           RST_QPLL <= 1;
        end
        Pow_on_Rst:        por_cnt <= 1;
        Run_State :        POR <= 0;
        W4Qpll    :        MMCM_RST <= 1;
      endcase
    end
  end
  
  assign por_done   = por_cnt && (timer == POR_tmo);

always @(posedge CLK)
begin
   if(por_cnt)
      if(por_done)
         timer <= timer;
      else
         timer <= timer + 1;
   else
      timer <= 4'h0;
end

  // This code allows you to see state names in simulation
  `ifndef SYNTHESIS
  reg [79:0] statename;
  always @* begin
    case (state)
      Qpll_rst  : statename = "Qpll_rst";
      Pow_on_Rst: statename = "Pow_on_Rst";
      Run_State : statename = "Run_State";
      W4Qpll    : statename = "W4Qpll";
      W4SysClk  : statename = "W4SysClk";
      default   : statename = "XXXXXXXXXX";
    endcase
  end
  `endif

endmodule


////////////////////////////////////////////////////////////////
//                                                            //
// Trigger Clock Startup  State Machine       (Uses STUP_CLK) //
//                                                            //
////////////////////////////////////////////////////////////////

module Trg_Clock_Strt (
  output reg GTX_RST,
  output reg TRG_RST,
  input CLK,
  input MMCM_LOCK,
  input POR,
  input RST_DONE 
);
  
  // state bits
  parameter 
  GTX_Idle = 2'b00, 
  Clk_Run  = 2'b01, 
  W4Lock   = 2'b10; 
  
  reg [1:0] state;
  reg [1:0] nextstate;
  
 
  // comb always block
  always @* begin
    nextstate = 2'bx; // default to x because default_state_is_x is set
    case (state)
      GTX_Idle:                            nextstate = W4Lock;
      Clk_Run : if (!MMCM_LOCK)            nextstate = GTX_Idle;
                else                       nextstate = Clk_Run;
      W4Lock  : if (MMCM_LOCK & RST_DONE)  nextstate = Clk_Run;
                else                       nextstate = W4Lock;
    endcase
  end
  
  // Assign reg'd outputs to state bits
  
  // sequential always block
  always @(posedge CLK or posedge POR) begin
    if (POR)
      state <= GTX_Idle;
    else
      state <= nextstate;
  end
  
  // datapath sequential always block
  always @(posedge CLK or posedge POR) begin
    if (POR) begin
      GTX_RST <= 1;
      TRG_RST <= 1;
    end
    else begin
      GTX_RST <= 1; // default
      TRG_RST <= 1; // default
      case (nextstate)
        Clk_Run : begin
                         GTX_RST <= 0;
                         TRG_RST <= 0;
        end
        W4Lock  :        GTX_RST <= 0;
      endcase
    end
  end
  
  // This code allows you to see state names in simulation
  `ifndef SYNTHESIS
  reg [63:0] statename;
  always @* begin
    case (state)
      GTX_Idle: statename = "GTX_Idle";
      Clk_Run : statename = "Clk_Run";
      W4Lock  : statename = "W4Lock";
      default : statename = "XXXXXXXX";
    endcase
  end
  `endif

endmodule


////////////////////////////////////////////////////////////////
//                                                            //
// ADC Initialization  State Machine             (Uses CLK40) //
//                                                            //
////////////////////////////////////////////////////////////////


// Created by fizzim.pl version 3.60 on 2011:04:13 at 16:07:08 (www.fizzim.com)

module ADC_Init (
  output reg INC,
  output reg INIT,
  output reg RST,
  output reg RUN,
  input CLK,
  input wire [2:0] CNT,
  input INIT_DONE,
  input POR 
);

  // state bits
  parameter 
  Reset = 2'b00, 
  Init  = 2'b01, 
  Run   = 2'b10, 
  Wait  = 2'b11; 
  
  reg [1:0] state;
  reg [1:0] nextstate;
  
  // comb always block
  always @* begin
    nextstate = 2'bx; // default to x because default_state_is_x is set
    case (state)
      Reset:                 nextstate = Wait;
      Init : if (INIT_DONE)  nextstate = Run;
             else            nextstate = Init;
      Run  :                 nextstate = Run;
      Wait : if (CNT == 7)   nextstate = Init;
             else            nextstate = Wait;
    endcase
  end
  
  // Assign reg'd outputs to state bits
  
  // sequential always block
  always @(posedge CLK or posedge POR) begin
    if (POR)
      state <= Reset;
    else
      state <= nextstate;
  end
  
  // datapath sequential always block
  always @(posedge CLK or posedge POR) begin
    if (POR) begin
      INC <= 0;
      INIT <= 0;
      RST <= 1;
      RUN <= 0;
    end
    else begin
      INC <= 0; // default
      INIT <= 0; // default
      RST <= 0; // default
      RUN <= 0; // default
      case (nextstate)
        Reset: RST <= 1;
        Init : INIT <= 1;
        Run  : RUN <= 1;
        Wait : INC <= 1;
      endcase
    end
  end
  
  // This code allows you to see state names in simulation
  `ifndef SYNTHESIS
  reg [39:0] statename;
  always @* begin
    case (state)
      Reset:   statename = "Reset";
      Init :   statename = "Init";
      Run  :   statename = "Run";
      Wait :   statename = "Wait";
      default: statename = "XXXXX";
    endcase
  end
  `endif

endmodule

